-- Avatar
-- Added 2010-11-01 by Thomas Holden
-- Edited 2011-04-15 by Thomas Holden
-- Creates and governs the player avatar(s)

avatar = class:new()

function avatar:init(id, x, y, direction)
	self.id = id or 1
	self.name = "avatar"
	self.width = 64
	self.height = 64
	-- movement stuff
	self.direction = 1				-- 1 is right, -1 is left
	if direction == -1 then			-- we accept only two directions
		self.direction = -1
	end
	self.wallJump = 0							-- which way are we walljumping
	self.jumpsLeft = players[self.id].max.jumps	-- can avatar still jump
	-- physics stuff
	self.buoyancy = 240
	self.swimming = false
	self.body = love.physics.newBody(world, x, y, 10, 0)
	self.body:setBullet(true)		-- more accurate collision detection
	self.shape = love.physics.newCircleShape(self.body, 0, 0, self.width / 2)
	self.shape:setCategory(1)
	self.shape:setData(self)
	-- graphics stuff
	local animSheet = loadAnimSheet("actors", players[self.id].species.."-run")
	self.anim = newAnimation(animSheet, self.width, self.height, .1, 4)
	self.image = newPaddedImage("media/images/actors/"..players[self.id].species.."-jump.png")
	self.animSpeeds = { .3, .7, 1 }		-- different speeds for the run animation
	printInit(self, x, y)
end

function avatar:update(dt)
	local xVel, yVel = self.body:getLinearVelocity()
	--cap avatar's speed
	if math.abs(xVel) > players[self.id].max.speed then
		self.body:setLinearVelocity(players[self.id].max.speed * self.direction, yVel)
	end
	if self.wallJump ~= 0 then
		self.wallJump = self.wallJump - dt
	end
	--update the animation
	if math.abs(xVel) <= 300 and self.anim:getSpeed() ~= self.animSpeeds[1] then
		self.anim:setSpeed(self.animSpeeds[1])
	elseif math.abs(xVel) > 300 and math.abs(xVel) <= 500 and self.anim:getSpeed() ~= self.animSpeeds[2] then
		self.anim:setSpeed(self.animSpeeds[2])
	elseif math.abs(xVel) > 500 and self.anim:getSpeed() ~= self.animSpeeds[3] then
		self.anim:setSpeed(self.animSpeeds[3])
	end
	self.anim:update(dt)
	-- killdepth handling
	if self.body:getY() > killdepth and players[self.id].health > 0 then
		print("Avatar ID " .. self.id .. " has dropped below the killdepth")
		if editor.state then
			self.body:setPosition(8192, 8192)
		elseif not scene.ending then
			self:kill()
			players[self.id].health = 0
		end
	end
	-- keep avatar from going outside the world
	if self.body:getX() < 32 then
		self.body:setX(32)
		self.body:setLinearVelocity(0, yVel)
	end
	if self.body:getY() < 32 then
		self.body:setY(32)
		self.body:setLinearVelocity(xVel, 0)
	end
end

function avatar:draw()
	local drawX = self.body:getX()
	local drawY = self.body:getY()
	if players[self.id].team == 2 then
		love.graphics.setColor(255, 96, 96, 255)
	end
	if players[self.id].health > 0 then
		if self.jumpsLeft < players[self.id].max.jumps then	-- we're probably mid-jump
			love.graphics.draw(self.image, drawX, drawY, 0, self.direction, 1, 32, 32)
		else
			local xVel, yVel = self.body:getLinearVelocity()
			if xVel > -0.1 and xVel < 0.1 then	-- just standing there
				self.anim:stop()
			else	-- we're moving at least a little
				self.anim:play()
			end
			self.anim:draw(drawX, drawY, 0, self.direction, 1, 32, 32)
		end
	else	-- we're dead
		self.anim:draw(drawX, drawY, 0, self.direction, 1, 32, 32)
	end
	setDefaultColor()
end

function avatar:collide(shapeData)
	local jumpables = {"avatar", "baddie", "boss", "blockMaker", "breakable", "crate", "madeBlock", "platform", "ramp", "solidBlock", "terrain", "barrier"}
	local theirX, theirY = shapeData.body:getPosition()
	local theirWidth = shapeData.width / 2
	local theirHeight = shapeData.height / 2
	local theirCat = shapeData.shape:getCategory()
	local selfX, selfY = self.body:getPosition()
	local selfWidth = self.width / 4
	local selfHeight = self.height / 4
	if testStrings(shapeData.name, jumpables) and 
		(theirY > selfY and theirY - theirHeight >= selfY + selfHeight) and
		(theirX - theirWidth < selfX + selfWidth or theirX + theirWidth > selfX - selfWidth)
		then
		self.jumpsLeft = players[self.id].max.jumps	-- we landed, restock jumps
		self.wallJump = 0
	elseif theirY + theirHeight >= selfY - selfHeight then		-- 
		if self.jumpsLeft < players[self.id].max.jumps and theirCat == 4 then
			if theirX < selfX then
				self.wallJump = 1	-- we hit a wall to the left, so we can walljump right
			elseif theirX > selfX then
				self.wallJump = -1	-- we hit a wall to the right, so we can walljump left
			end
		end
	end
end

function avatar:kill()
	print("Killing Avatar ID " .. self.id)
	players[self.id].health = 0
	self.anim:stop()
	self.shape:setMask(1,2,3,4)
	self.body:applyImpulse(0, -64)
	sfx:play("death")
end

function avatar:run(force)
	force = force or 50
	if self.swimming then
		force = 25
	end
	local xVel, yVel = self.body:getLinearVelocity()
	if self.jumpsLeft == players[self.id].max.jumps then
		if math.abs(xVel) < 10 then 	-- fixes bug where avatar won't move in certain spots
			self.body:setLinearVelocity(50 * self.direction, yVel)
		end
		self.body:applyForce(5 * force * self.direction, 0)
	else
		self.body:applyForce(2 * force * self.direction, 0)		-- can't move as much in midair
	end
end

function avatar:lunge(impulse)
	impulse = impulse or 200
	self.body:applyImpulse(impulse * self.direction, 0)
	print("Avatar ID " .. self.id .. " lunged")
end

function avatar:duck(impulse)
	impulse = impulse or 50
	self.body:applyImpulse(0, impulse)
	print("Avatar ID " .. self.id .. " ducked")
end

function avatar:jump(impulse)
	impulse = impulse or 100
	local xVel, yVel = self.body:getLinearVelocity()
	if math.abs(xVel) > 600 then	-- set the jump height depending on speed
		impulse = 120
	end
	if math.abs(yVel) > 100 then	-- second+ jump has less effect unless you time it a little
		impulse = 80
	end
	if self.swimming then
		self.jumpsLeft = players[self.id].max.jumps + 1	-- infinite jumpage!
		impulse = 110
	end
	if self.wallJump == 0 then -- regular jump
		if self.jumpsLeft > 0 then
			self.body:applyImpulse(0, -impulse)
		end
	else
		if (self.wallJump > .5) or (self.wallJump <= -1 and self.wallJump > -1.5) then
			self.direction = math.ceil(self.wallJump)
			self.body:setLinearVelocity(400 * self.direction, -640)	-- walljump is notably more powerful
		end
	end
	self.jumpsLeft = self.jumpsLeft - 1
	self.wallJump = 0
	sfx:play("jump")
	print("Avatar ID " .. self.id .. " jumped")
end

function avatar:setHealth(number, cause)
	players[self.id]:setHealth(number, cause)
end
