-- Baddie
-- Added 2010-11-01 by Thomas Holden
-- Edited 2011-03-04 by Thomas Holden
-- Creates and governs the generic enemy

baddie = class:new()

function baddie:init(x, y, width, height, species, stats, drop, buoyancy, frames, fps)
	self.id = 1
	self.name = "baddie"
	self.width = width or 32
	self.height = height or 32
	self.species = species or "baddie"
	self.motion = {
		direction = -1,
		buoyancy = buoyancy or 240,
		last_change = 0,
		canJump = false		-- unused
	}
	stats = stats or {32, 1, 1}
	self.stats = {
		speed = stats[1],
		strength = stats[2],
		health = stats[3]
	}
	self.drop = drop or "attack"
	self.body = love.physics.newBody(world, x + (self.width / 2), y + (self.height / 2))
	-- create a polygon with 1/4 chipped corners
	local qW = self.width / 4
	local qH = self.height / 4
	self.shape = love.physics.newPolygonShape(self.body, -2*qW,-qH, -qW,-2*qH, qW,-2*qH, 2*qW,-qH, 2*qW,qH, qW,2*qH, -qW,2*qH, -2*qW,qH)
	self.shape:setCategory(1)
	if editor.state then
		self.shape:setSensor(true)
	else
		self.body:setAngularDamping(100)
		self.body:setMassFromShapes()
		self.body:putToSleep()
	end
	self.cane = cane:new(self.id, x, y, self.width, self.height)
	self.cane.shape:setMask(1, 2, 3)
	local animSheet = loadAnimSheet("actors", self.species)
	self.anim = newAnimation(animSheet, self.width, self.height, (fps or .35), (frames or 2))
	self.sleeping = true
	self.shape:setData(self)
	printInit(self, x, y)
end

function baddie:update(dt)
	if self.stats.health > 0 then
		if self.sleeping then
			local cullX = self.body:getX() - (self.width / 2)
			local cullY = self.body:getY() - (self.height / 2)
			if testOverlap(cullX, cullY, self.width, self.height) then
				self.sleeping = false
				self.body:wakeUp()
				self.body:setLinearVelocity(self.stats.speed * self.motion.direction, 0)
			end
		else
			local xVel, yVel = self.body:getLinearVelocity()
			self.body:setLinearVelocity(self.stats.speed * self.motion.direction, yVel)
			self.body:setAngle(0)		-- keeps us from "tipping" on ledges
			self.anim:update(dt)
			self.cane:update(dt)
			if self.cane.clock > (32 / self.stats.speed) then
				self.cane:reposition(self.body, self.width, self.motion.direction)
			end
			if self.cane.body:getY() > self.body:getY() + (self.height / 2) then
				self:changeDirection()
			end
		end
		self.motion.last_change = self.motion.last_change + dt
	end
end

function baddie:draw()
	local cullX = self.body:getX() - (self.width / 2)
	local cullY = self.body:getY() - (self.height / 2)
	if testOverlap(cullX, cullY, self.width, self.height) then
		local scaleY = 1
		if self.stats.health <= 0 then
			scaleY = -1		-- squish the baddie
		end
		self.anim:draw(self.body:getX(), self.body:getY(), self.body:getAngle(), self.motion.direction, scaleY, (self.width / 2), (self.height / 2))
		if editor.state and self.cane then
			self.cane:draw()
		end
	end
end

function baddie:collide(shapeData)
	if not editor.state then
		local theirX, theirY = shapeData.body:getPosition()
		--local theirWidth = shapeData.width / 2
		local theirHeight = shapeData.height / 2
		local selfX, selfY = self.body:getPosition()
		--local selfWidth = self.width / 2
		local selfHeight = self.height / 2
		if shapeData.name == "avatar" then
			if theirY + theirHeight < selfY - (selfHeight / 4) then	-- their bottom edge is above our top edge, give or take
				local xVel, yVel = shapeData.body:getLinearVelocity()
				shapeData.body:setLinearVelocity(xVel, -200) -- give the player some bounce after he steps on the baddie
				self:setHealth(-1, shapeData)
				sfx:play("stomp")
			else
				self:attack(shapeData)
			end
		elseif shapeData.name == "clip" then
			self:changeDirection()
		else
			if round(theirY - theirHeight) < round(selfY + selfHeight) then		-- hit a ledge
				self:changeDirection()
			end
		end
	end
end

function baddie:kill()
	print("Killing Baddie ID " .. self.id)
	local deadX, deadY = self.body:getPosition()
	self.shape:setMask(1,2,3,4)
	if self.drop ~= " " then
		ent.sensor.pickup[#ent.sensor.pickup + 1] = pickup:new(deadX, deadY - (self.height / 2), self.drop, 1)		-- drop an item
		ent.sensor.pickup[#ent.sensor.pickup].id = #ent.sensor.pickup
		print("Baddie ID " .. self.id .. " dropped a " .. self.drop .. " pickup")
	end
end

function baddie:attack(shapeData)
	print("Baddie ID " .. self.id .. " is attacking player " .. shapeData.id)
	local xVel, yVel = shapeData.body:getLinearVelocity()
	if shapeData.body:getX() < self.body:getX() then 	-- bounce the player to side so he doesn't die in three frames
		shapeData.body:setLinearVelocity(-200, yVel)
	else 
		shapeData.body:setLinearVelocity(200, yVel)
	end
	players[shapeData.id]:setHealth(-self.stats.strength, self)
	self:changeDirection()
end

function baddie:changeDirection()
	if self.motion.last_change > 1 then		-- only change direction once per second
		self.motion.direction = -self.motion.direction
		self.motion.last_change = 0
		self.cane:reposition(self.body, self.width, self.motion.direction)
		print("Baddie ID " .. self.id .. " changed direction")
	end
end

function baddie:setHealth(number, cause)
	self.stats.health = self.stats.health + number
	print("Baddie ID " .. self.id .. " Health = " .. self.stats.health)
	if self.stats.health <= 0 then
		if testStrings(cause.name, {"avatar", "bomb", "boomerang", "lightning"}) then
			players[cause.id].history.kills = players[cause.id].history.kills + 1
		end
		self:kill()
	end
end
