-- Boss
-- Added 2010-11-22 by Thomas Holden
-- Edited 2011-03-04 by Thomas Holden
-- Creates and governs a boss enemy

boss = class:new()

function boss:init(x, y, species, width, height, stats, drop, buoyancy, frames, fps)
	self.id = 1
	self.name = "boss"
	self.species = species or "boss"
	self.width = width or 256
	self.height = height or 128
	self.motion = {
		direction = -1,			-- 1 is right, -1 is left
		buoyancy = buoyancy or 240,
		last_change = 0,
		canJump = false		-- unused
	}
	stats = stats or {16, 2, 6}
	self.stats = {
		speed = stats[1],
		strength = stats[2],
		health = stats[3]
	}
	self.drop = drop or "will"
	self.body = love.physics.newBody(world, x + (self.width / 2), y + (self.height / 2))
	self.shape = love.physics.newRectangleShape(self.body, 0, 0, self.width, self.height)
	self.shape:setCategory(1)
	if editor.state then
		self.shape:setSensor(true)
	else
		self.body:setMassFromShapes()
		self.body:putToSleep()
	end
	self.cane = cane:new(self.id, x, y, self.width, self.height)
	self.cane.shape:setMask(1, 2, 3)
	local animSheet = loadAnimSheet("actors", self.species.."_body")
	self.bodyAnim = newAnimation(animSheet, (self.width / 2), self.height, (fps or .25), (frames or 4))
	animSheet = loadAnimSheet("actors", self.species.."_legs")
	self.legAnim = newAnimation(animSheet, ((self.width / 2) + 32), (self.height + 32), (fps or .25), (frames or 4))
	self.sleeping = true
	self.shape:setData(self)
	printInit(self, x, y)
end

function boss:update(dt)
	if self.stats.health > 0 then
		if self.sleeping then
			local cullX = self.body:getX() - (self.width / 2)
			local cullY = self.body:getY() - (self.height / 2)
			if testOverlap(cullX, cullY, self.width, self.height) then
				self.sleeping = false
				self.body:wakeUp()
			end
		else
			local xVel, yVel = self.body:getLinearVelocity()
			self.body:setLinearVelocity(self.stats.speed * self.motion.direction, yVel)
			self.body:setAngle(0)	-- keeps us from "tipping" on ledges
			self.bodyAnim:update(dt)
			self.legAnim:update(dt)
			self.cane:update(dt)
			if self.cane.clock > (32 / self.stats.speed) then
				self.cane:reposition(self.body, self.width, self.motion.direction)
			end
			if self.cane.body:getY() > self.body:getY() + (self.height / 2) then
				self:changeDirection()
			end
			self.motion.last_change = self.motion.last_change + dt
		end
	end
end

function boss:draw()
	local cullX = self.body:getX() - (self.width / 2)
	local cullY = self.body:getY() - (self.height / 2)
	if testOverlap(cullX, cullY, self.width, self.height) then
		local drawX = self.body:getX()
		local drawY = self.body:getY()
		local scaleY = 1
		if self.stats.health <= 0 then
			scaleY = -1
		end
		self.legAnim:draw(drawX, drawY, 0, 1, scaleY, ((self.width / 2) + 32), (self.height - 32))		-- left legs
		self.legAnim:draw(drawX, drawY, 0, -1, scaleY, ((self.width / 2) + 32), (self.height - 32))		-- right legs
		self.bodyAnim:draw(drawX, drawY, 0, 1, scaleY, (self.width / 4), (self.height / 2))
		if editor.state and self.cane then
			self.cane:draw()
		end
	end
end

function boss:collide(shapeData)
	if not editor.state then
		if shapeData.name == "avatar" then
			self:attack(shapeData)
		elseif shapeData.name == "clip" then
			self:changeDirection()
		else
			local theirX, theirY = shapeData.body:getPosition()
			--local theirWidth = shapeData.width / 2
			local theirHeight = shapeData.height / 2
			local selfX, selfY = self.body:getPosition()
			--local selfWidth = self.width / 2
			local selfHeight = self.height / 2
			if theirY - theirHeight < selfY + selfHeight + 2 then
				self:changeDirection()
			end
		end
	end
end

function boss:kill()
	print("Killing Baddie ID " .. self.id)
	local deadX, deadY = self.body:getPosition()
	self.shape:setMask(1,2,3,4)
	if self.drop ~= " " then
		ent.sensor.pickup[#ent.sensor.pickup + 1] = pickup:new(deadX, deadY - (self.height / 2), self.drop)		-- drop an item (unlock?)
		ent.sensor.pickup[#ent.sensor.pickup].id = #ent.sensor.pickup
		print("Boss ID " .. self.id .. " dropped a " .. self.drop .. " pickup")
	end
end

function boss:attack(shapeData)
	local xVel, yVel = shapeData.body:getLinearVelocity()
	if shapeData.body:getX() < self.body:getX() then 	-- more powerful kickback than the baddie
		shapeData.body:setLinearVelocity(-300, yVel)
	else 
		shapeData.body:setLinearVelocity(300, yVel)
	end
	players[shapeData.id]:setHealth(-self.stats.strength, self)
	-- note: boss does not change direction after an attack, they press the offensive
end

function boss:changeDirection()
	if self.motion.last_change > 1 then		-- only change direction once per second
		self.motion.direction = -self.motion.direction
		self.motion.last_change = 0
		self.cane:reposition(self.body, self.width, self.motion.direction)
		print("Boss ID " .. self.id .. " changed direction")
	end
end

function boss:setHealth(number, cause)
	self.stats.health = self.stats.health + number
	if self.stats.health <= 0 then
		if testStrings(cause.name, {"avatar", "bomb", "boomerang", "lightning"}) then
			players[cause.id].history.kills = players[cause.id].history.kills + 1
		end
		self:kill()
	end
end
