-- Game
-- Added 2010-11-01 by Thomas Holden
-- Edited 2011-03-03 by Thomas Holden
-- Creates and renders the level, avatars, hud, etc

game = class:new()

function game:init()
	currentScene = "game"
	require "ents/requires.lua"
	fader:start(255, -128)
	menuList:reposition((screen.halfWidth - 128), (screen.halfHeight - 128))
	menuList.bg = true
	self.startX = {}
	self.startY = {}
	self.pause = false
	self.ending = false
	if gamemode == "capture" or gamemode == "team_capture" then
		loadStaticImg("actors", "bucket")
		self.score = {0,0}
		if gamemode == "capture" and multiplayer == 3 then
			self.score = {0,0,0}
		elseif gamemode == "capture" and multiplayer == 4 then
			self.score = {0,0,0,0}
		end
	end
	self.playersLiving = multiplayer
	loadStaticImg("world", "bg")
	self:loadLevel()
	print("Game scene initiated")
end

function game:update(dt)
	if not self.pause then
		if not dialog.state then
			world:update(dt)
			-- update the players
			for i=1, multiplayer do
				if players[i].health > 0 then
					players[i]:update(dt)
				else	-- you instantly respawn in capture mode
					if gamemode == "capture" or gamemode == "team_capture" then
						self:respawnPlayer(i)
					end
				end
			end
			-- update the actors
			if not editor.state then
				self:updateLoop({"actor"}, dt)
			else	-- in the editor we do not update the other actors, just the player avatar
				ent.actor.avatar[1]:update(dt)
			end
			self.playersLiving = self:countLiving()
			self:updateLoop({"projectile", "sensor", "geometry"}, dt)
			-- update camera position
			camera:update(dt)
		end
	else
		menuList:update(dt)
	end
	joystick:update(dt)
	fader:update(dt)
end

function game:draw()
	love.graphics.draw(staticImg.bg, screen.width - 512, 0)
	love.graphics.push()
	camera:draw()
	-- draw all the world ents in a specific order
	self:drawLoop({"geometry", "sensor", "actor", "projectile"})
	love.graphics.pop()
	-- gamemode specific renders
	if gamemode == "versus" then
		if self.playersLiving <= 1 then		-- if only one player is left standing
			if not self.ending then
				versusRecordResults()
				self.ending = true
			end
			love.graphics.printf(" Victory! \n Press enter to start again. ", 0, (screen.halfHeight - 32), screen.width, "center")
		end
	elseif gamemode == "team_versus" then
		local team1, team2 = teamCheckLiving()
		if (team1 and not team2) or (team2 and not team1) then		-- if only one team is left standing
			local winner = 1		-- assume team 1 won
			if team2 then			-- nope, it was team 2
				winner = 2
			end
			if not self.ending then
				teamRecordResults(winner)
				self.ending = true
			end
			love.graphics.printf(" Team " .. winner .. " is victorious! \n Press enter to start again. ", 0, (screen.halfHeight - 32), screen.width, "center")
		end
	elseif gamemode == "capture" or gamemode == "team_capture" then
		local currentscore = self.score[1]
		for i=2, #self.score do
			currentscore = currentscore .. "  vs  " .. self.score[i]
		end
		love.graphics.printf(currentscore, 0, 8, screen.width, "center")
	else	-- singleplayer, coop
		if self.playersLiving < 1 and not self.ending then		-- if no one is alive and an eol hasn't been hit
			love.graphics.printf(" You have perished. \n Press enter to continue. ", 0, (screen.halfHeight - 32), screen.width, "center")
		end
	end
	if dialog.state then
		dialog:draw()
	end
	if editor.state then
		editor:draw()
	end
	fader:draw()
	if self.pause then
		menuList:draw()
		if menuList.gui.key == "stats_menu" then
			for i=1, multiplayer do
				players[i]:drawStats()
			end
		end
	end
	if showFPS then
		love.graphics.print(love.timer.getFPS(), screen.width - 32, screen.height - 24)
	end
end

function game:loadLevel()
	print("Loading level")
	song = "level1"		-- default to first level's song
	self:createWorld()
	self:createEntTables()	-- will also remove any holdover avatars, projectiles, etc from last level
	if editor.state then
		if currentLevel ~= "new" then		-- editing a map that already exists
			love.filesystem.load("userlevels/"..currentLevel..".lua")()		-- load user level
		else 	-- making a new map
			killdepth = 16000
			ent.geometry.terrain[1] = terrain:new(8000, 8064, 8, 2)
		end
		players[1].max.jumps = 10000	-- we can essentially fly in edit mode
	else
		love.filesystem.load("levels/"..currentLevel..".lua")()		-- load level
	end
	local list = {"geometry", "sensor", "actor", "projectile"}
	for i=1, #list do
		for key, value in pairs(ent[list[i]]) do
			setIDs(list[i], key)
		end
	end
	self:respawnAll()	-- will manually set player IDs later
	music:stop()
	music:play(song)
end

function game:createEntTables()
	print("Creating ent tables")
	ent = {
		actor = {		-- actors are active, moving ents with mass and rudimentary AI
			avatar = {},
			companion = {},
			baddie = {},
			boss = {},
			npc = {},
			signpost = {},
			floater = {}
		},
		sensor = {		-- sensors are ents that react to the actors but don't have mass
			charge = {},
			eol = {},
			home = {},
			pickup = {},
			teleport = {},
			spring = {},
			unlock = {}
		},
		projectile = {		-- projectiles are basically sensor ents but have some special rules
			lightning = {},
			bomb = {},
			boomerang = {}
		},
		geometry = {		-- geometry: fixed-position ents that have mass
			blockMaker = {},
			breakable = {},
			clip = {},
			crate = {},
			barrier = {},
			nonsolid = {},
			platform = {},
			ramp = {},
			solidBlock = {},
			spikes = {},
			terrain = {},
			water = {},
			wellspring = {}
		}
	}
end

function game:createWorld(worldsize)
	print("Creating new physics world")
	worldsize = worldsize or 16384
	-- create the actual world
	world = love.physics.newWorld(0, 0, worldsize, worldsize)
	world:setGravity(0, 700)
	world:setCallbacks(add, persist, remove)
	world:setMeter(64)
end

function game:respawnAll()
	self.startX = {}
	self.startY = {}
	-- create avatars and companions
	for i=1, multiplayer do
		self:setStartPosition(i)
		self:respawnPlayer(i)
	end
end

function game:respawnPlayer(who)
	if players[who].health < 3 then	-- revigorate dead/hurt player
		players[who].health = 3
	end
	if ent.actor.avatar[who] then	-- hack to get around multiple body problems...
		physicsRemoval(ent.actor.avatar[who])
	end
	ent.actor.avatar[who] = avatar:new(who, self.startX[who], self.startY[who])
	ent.actor.avatar[who].id = who
	ent.actor.companion[who] = companion:new(who)
	if (gamemode == "versus" or gamemode == "team_versus") then		-- if we're in versus, everyone starts with level 1 moves
		versusSetStats(who, 1)
	end
end

function game:setStartPosition(who)
	print("Setting starting positions")
	local x = 8000 + (128 * who)		-- default starting coordinates
	local y = 8000
	if #ent.sensor.home > 0 then
		for j=1, #ent.sensor.home do
			if ent.sensor.home[j].targets[who] then
				x = ent.sensor.home[j].body:getX() + ent.sensor.home[j].targets[who]
				y = ent.sensor.home[j].body:getY() - 32
			end
		end
	end
	self.startX[who] = x
	self.startY[who] = y
end

function game:drawLoop(loopTargets)
	for i=1, #loopTargets do	-- loop numerically because order can be important
		local section = ent[loopTargets[i]]
		for key, value in pairs(section) do
			for j=1, #section[key] do
				section[key][j]:draw()
				if editor.state and editor.brush == section[key][j].name and editor.select == j then
					local width = section[key][j].width + 8
					local height = section[key][j].height + 8
					love.graphics.rectangle("line", section[key][j].body:getX() - (width / 2), section[key][j].body:getY() - (height / 2), width, height)
				end
			end
		end
	end
end

function game:updateLoop(loopTargets, dt)
	for i=1, #loopTargets do
		local section = ent[loopTargets[i]]
		for key, value in pairs(section) do
			for j=1, #section[key] do
				section[key][j]:update(dt)
			end
		end
	end
end

function game:countLiving()
	local total = 0
	for i=1, multiplayer do
		if players[i].health > 0 then
			total = total + 1
		end
	end
	return total
end

function game:keypressed(key, unicode)
	-- edit mode has its own set of special bindings
	if editor.state then
		editor:keypressed(key, unicode)
	end
	-- normal game bindings
	if self.pause then
		menuList:keypressed(key)
	elseif dialog.state then
		dialog:keypressed(key)
	else
		local playerNum, keyNum = checkBindings(key)
		if keyNum == 7 then
			self.pause = true	-- pause the game
		elseif playerNum > 0 and playerNum <= multiplayer then
			players[playerNum]:keypressed(key)
		end
		if key == "return" then
			if gamemode == "versus" or gamemode == "team_versus" then
				if self.playersLiving <= 1 then
					self:respawnAll()
				end
			else 
				if self.playersLiving < 1 then
					self:respawnAll()
				else
					-- recenter camera on player 1 at this moment
					local goalX = ent.actor.avatar[1].body:getX() - screen.halfWidth
					local goalY = ent.actor.avatar[1].body:getY() - screen.halfHeight
					camera:pan(nil, nil, goalX, goalY, 3)
				end
			end
		elseif key == "-" then
			camera.zoom.modifier = camera.zoom.modifier / 1.25
			print(camera.zoom.modifier)
		elseif key == "=" then
			if camera.zoom.modifier < 1 then
				camera.zoom.modifier = camera.zoom.modifier * 1.25
			end
			print(camera.zoom.modifier)
		elseif key == "f12" then	-- debugging god mode, kinda
			versusSetStats(1, 1)
		end
	end
end

function game:joystickpressed(stick, button)
	self:keypressed("j"..stick.."_b"..button)
end

function game:mousereleased(x, y, button)
	if editor.state then
		editor:mousereleased(x, y, button)
	end
end

function game:focus(f)
	if not f then
		print("Focus lost, pausing game")
		scene.pause = true
	end
end
