-- Camera
-- Added 2010-12-24 by Thomas Holden
-- Edited 2011-03-04 by Thomas Holden
-- Manages the "camera" object

camera = class:new() 

function camera:init()
	self.mode = ""
	self.x = 0			-- left edge of the camera
	self.y = 0			-- top edge	of the camera
	self.zoom = {
		x = 1,
		y = 1,
		modifier = 1
	}
	self.goal = {
		x = 0,
		y = 0
	}
	self.counter = 0
	self.rumble = {
		x = 0,
		y = 0,
		duration = 0
	}
end

function camera:draw()
	love.graphics.scale(self.zoom.x * self.zoom.modifier, self.zoom.y * self.zoom.modifier)
	local transX = -self.x
	local transY = -self.y
	if self.zoom.x ~= 1 or self.zoom.y ~= 1 or self.zoom.modifier ~= 1 then
		transX = transX + ((screen.width - (screen.width * self.zoom.x * self.zoom.modifier)) / 2)
		transY = transY + ((screen.height - (screen.height * self.zoom.y * self.zoom.modifier)) / 2)
	end
	if self.rumble.duration > 0 then
		self.rumble.x = -self.rumble.x
		self.rumble.y = -self.rumble.y
		transX = transX + (self.rumble.x * self.rumble.duration)
		transY = transY + (self.rumble.y * self.rumble.duration)
	end
	love.graphics.translate(transX, transY)
end

function camera:update(dt)
	if self.mode == "panning" then
		if self.counter > 0 then
			local step = self.counter * dt	-- this is a crude way to figure out the increment, but essentially works
			self.x = self.x - ((self.x - self.goal.x) * step)
			self.y = self.y - ((self.y - self.goal.y) * step)
			self.counter = self.counter - dt
		else
			self.x = self.goal.x
			self.y = self.goal.y
			self.counter = 0
			self.mode = ""
		end
	elseif self.mode == "waiting" then
		if self.counter > 0 then
			self.counter = self.counter - dt
		else
			self.counter = 0
			self.mode = ""
		end
	else
		if scene.playersLiving > 0 then
			if scene.playersLiving > 1 then
				self:updateZoom(dt)
			else
				self:changeZoom(1, 1)
			end
			self:updatePosition(dt)
		end
	end
	if self.rumble.duration > 0 then
		self.rumble.duration = self.rumble.duration - dt
	end
end

function camera:updateZoom(dt)
	local far = {}	-- the furthest points of each player
	for i=1, multiplayer do
		if players[i].health > 0 then
			-- find the most extreme coordinates of the living players
			if not far.right or ent.actor.avatar[i].body:getX() > far.right then
				far.right = ent.actor.avatar[i].body:getX()
			end
			if not far.left or ent.actor.avatar[i].body:getX() < far.left then
				far.left = ent.actor.avatar[i].body:getX()
			end
			if not far.up or ent.actor.avatar[i].body:getY() < far.up then
				far.up = ent.actor.avatar[i].body:getY()
			end
			if not far.down or ent.actor.avatar[i].body:getY() > far.down then
				far.down = ent.actor.avatar[i].body:getY()
			end
		end
	end
	local zoomout = {h=1, v=1}
	local hDist = (screen.width * .6)
	if (far.right - far.left) > hDist then
		zoomout.h = 1 / ((far.right - far.left) / hDist)
	end
	local vDist = (screen.height * .6)
	if (far.down - far.up) > vDist then
		zoomout.v = 1 / ((far.down - far.up) / vDist)
	end
	local newzoom = math.min(zoomout.h, zoomout.v)		-- whichever is smaller
	self:changeZoom(newzoom, newzoom)
end

function camera:updatePosition(dt)
	local mean = {
		x = 0,		-- average x position of the players
		y = 0,		-- average y
		count = 0	-- number of living players
	}
	for i=1, multiplayer do
		if players[i].health > 0 then		-- skip dead players, they don't need to see anyhow
			mean.x = mean.x + ent.actor.avatar[i].body:getX()
			mean.y = mean.y + ent.actor.avatar[i].body:getY()
			mean.count = mean.count + 1
		end
	end
	mean.x = mean.x / mean.count
	mean.y = mean.y / mean.count
	local margins = self:calcMargins()
	-- do we need to pan left or right
	if mean.x > self.x + margins.right then
		self.x = mean.x - margins.right
	elseif mean.x < self.x + margins.left then
		self.x = mean.x - margins.left
	end
	-- do we need to pan up or down
	if mean.y > self.y + margins.top then
		self.y = mean.y - margins.top
	elseif mean.y < self.y + margins.foot then
		self.y = mean.y - margins.foot
	end
end

function camera:calcMargins()
	local horizontal = (screen.width / self.zoom.x)
	local vertical = (screen.height / self.zoom.y)
	local margins = {
		right = horizontal * .6,
		left = horizontal * .4,
		top = vertical * .6,
		foot = vertical * .4
	}
	return margins
end

function camera:changeZoom(x, y, modifier)
	self.zoom.x = x or 1
	self.zoom.y = y or 1
	self.zoom.modifier = modifier or self.zoom.modifier
end

function camera:wait(x, y, counter)
	self.mode = "waiting"
	self.x = x or self.x
	self.y = y or self.y
	self.counter = counter or self.counter
end

function camera:pan(x, y, goalX, goalY, counter, mode)
	self.mode = mode or "panning"
	self.x = x or self.x
	self.y = y or self.y
	self.goal.x = goalX or self.goal.x
	self.goal.y = goalY or self.goal.y
	self.counter = counter or self.counter
end

function camera:shake(x, y, duration)
	self.rumble.x = x or 1
	self.rumble.y = y or 1
	self.rumble.duration = duration or 1
end
