-- editor
-- Added 2011-01-02 by Thomas Holden
-- Edited 2011-04-16 by Thomas Holden
-- controls various editing functions and whatnot

editor = class:new()

function editor:init()
	self.state = false
	self.select = 1			-- the id of what we have selected
	self.brush = "terrain"	-- the string for our current brush
	self.set = "geometry"	-- which set we're using in the brushList
	self.brushIndex = 1		-- the index for our current brush set
	self.brushList = {		-- the list of strings used in changing brushes
		geometry = {
			"terrain",
			"barrier",
			"blockMaker",
			"breakable",
			"clip",
			"crate",
			"platform",
			"solidBlock",
			"spikes",
			"water",
			"wellspring"
		},
		sensor = {
			"charge",
			"eol",
			"home",
			"pickup",
			"spring",
			"teleport",
			"unlock"
		},
		actor = {
			"baddie",
			"boss",
			"npc",
			"floater",
			"signpost"
		}
	}
	self.modifier = 1
	self.mouse = newPaddedImage("media/images/gui/mouse.png")
end

function editor:update(dt)
	-- not much
end

function editor:draw()
	local lineY = killdepth - camera.y
	if lineY <= screen.height then	-- show the killdepth
		love.graphics.rectangle("fill", 0, lineY, screen.width, 2)
	end
	if editHelp then	-- give a little help to people in edit mode
		self:drawEditHelp()
		self:drawDetails()
	else
		love.graphics.print(self.brush, 8, 8)
	end
	love.graphics.print("Modifier = "..self.modifier.."x", 8, 32)
	-- draw the mouse
	love.graphics.draw(self.mouse, love.mouse.getX() - 8, love.mouse.getY() - 8)
end

function editor:keypressed(key, unicode)
	if key == "kp5" or key == " " then	-- create terrain
		self:newObject()
	elseif key == "delete" then
		self:deleteEnt()
	elseif key == "lshift" then
		self.modifier = self.modifier * 2
		if self.modifier > 8 then
			self.modifier = 1
		end
	elseif key == "f4" or key == "f5" then
		if self.set == "geometry" then
			local ox, oy = self:getOffsetMousePosition()
			if key == "f4" then
				self:fillArea(modDown(ox), modDown(oy), 4, 2048)
			else
				self:fillArea(modDown(ox), modDown(oy), 8, 4096)
			end
		else
			dialog:setText("This function does not support actors or sensors.")
			dialog:setResponses()
		end
	elseif key == "kp0" or key == "lctrl" then	-- change brush set
		self:changeBrushSet()
	elseif key == "kp+" or key == "tab" then	-- change selection
		self:changeSelection(1)
	elseif key == "kp-" or key == "caps" then
		self:changeSelection(-1)
	elseif key == "kp*" or key == "f" then
		self:changeBrush(1)
	elseif key == "kp/" or key == "r" then
		self:changeBrush(-1)
	elseif testStrings(key, {"kp4","kp6","kp8","kp2", "a","d","w","s"}) then	-- move terrain
		local distance = 32 * self.modifier
		if key == "kp4" or key == "a" then
			self:moveObject(-distance, 0)
		elseif key == "kp6" or key == "d" then
			self:moveObject(distance, 0)
		elseif key == "kp8" or key == "w" then
			self:moveObject(0, -distance)
		elseif key == "kp2" or key == "s" then
			self:moveObject(0, distance)
		end
	elseif testStrings(key, {"kp7","kp9","kp1","kp3", "q","e","z","c"}) then	-- resize terrain
		if self.set == "geometry" then
			if self.brush == "blockMaker" or self.brush == "platform" or self.brush == "wellspring" or self.brush == "barrier" then
				dialog:setText("This brush cannot be scaled.")
				dialog:setResponses()
			else
				local amount = 32 * self.modifier
				if key == "kp7" or key == "q" then
					self:scaleObject(-amount, 0)
				elseif key == "kp3" or key == "c" then
					self:scaleObject(amount, 0)
				elseif key == "kp1" or key == "z" then
					self:scaleObject(0, -amount)
				elseif key == "kp9" or key == "e" then
					self:scaleObject(0, amount)
				end
			end
		elseif self.set == "sensor" then
			if self.brush == "pickup" then
				editor:changeSensorParams("category", 1)
			else
				dialog:setText("This brush's parameters cannot be changed.")
				dialog:setResponses()
			end
		else
			editor:changeActorParams("species", 1)
		end
	end
end

function editor:mousereleased(x, y, button)
	print("Mouse button " .. button .. " released")
	local ox, oy = self:getOffsetMousePosition()
	if button == "l" then
		self:newEnt(ox, oy, 32)
	elseif button == "m" then
		self:changeBrushSet()
	elseif button == "r" then
		self:changeSelectionViaMouse(ox, oy)
	elseif button == "wd" then
		self:changeBrush(1)
	elseif button == "wu" then
		self:changeBrush(-1)
	end
end

function editor:startup()
	editor.state = true
	self.select = 1
	self.brush = "terrain"
	self.set = "geometry"
	self.brushIndex = 1
	self.modifier = 1
end

function editor:getOffsetMousePosition()
	local x, y = love.mouse.getPosition( )
	if camera.zoom.x ~= 1 or camera.zoom.y ~= 1 or camera.zoom.modifier ~= 1 then
		local zoom = camera.zoom.x * camera.zoom.modifier
		x = (x / zoom) - ((screen.width - (screen.width * zoom)) / 2)
		zoom = camera.zoom.y * camera.zoom.modifier		-- redundant ATM
		y = (y / zoom) - ((screen.height - (screen.height * zoom)) / 2)
	end
	x = camera.x + x
	y = camera.y + y
	print("Returning mouse position at " .. x .. " by " .. y)
	return x, y
end

function editor:drawEditHelp()
	if currentLevel == "new" then
		love.graphics.print("You making a new level (hit ESC and select 'Save Level' to save).", 8, 8)
	else
		love.graphics.print("You are editing "..currentLevel..".", 8, 8)
	end
	local x = 0
	for key, value in pairs(self.brushList) do
		local brushText = key..":\n"
		if key ~= self.set then
			love.graphics.setColor(255, 255, 255, 160)
		end
		for j=1, #self.brushList[key] do
			if j == self.brushIndex and key == self.set then
				brushText = brushText .. "> "
			else 
				brushText = brushText .. "  "
			end
			brushText = brushText .. self.brushList[key][j] .. "\n"
		end
		love.graphics.print(brushText, (x * 128) + 8, 64)
		setDefaultColor()
		x = x + 1
	end
end

function editor:drawDetails()
	if self.select > 0 then
		local subject = ent[self.set][self.brush][self.select]
		local details = self:getDetails(self.brush)
		local height = (#details + 1) * 28
		love.graphics.setColor(0, 0, 0, 192)
		love.graphics.rectangle("fill", 0, screen.height - height, 256, height)
		setDefaultColor()
		love.graphics.print("Body X/Y: "..subject.body:getX() .. " x " .. subject.body:getY(), 8, screen.height - height)
		for i=1, #details do
			local data = toText(subject[details[i]])
			love.graphics.print(details[i]..": "..data, 8, screen.height - ((#details - i + 1) * 28))
		end
	end
end

function editor:getDetails(what)
	if what == "barrier" then
		return {"visible"}
	elseif what == "blockMaker" then
		return {"xchange", "ychange", "interval"}
	elseif what == "breakable" or what == "solidBlock" or what == "terrain" or what == "water" then
		return {"width", "height", "visible"}
	elseif what == "clip" then
		return {"width", "height"}
	elseif what == "crate" then
		return {"rotation"}
	elseif what == "platform" then
		return {"speed", "mode"}
	elseif what == "ramp" then
		return {"width", "height", "direction", "visible"}
	elseif what == "spikes" then
		return {"width", "height", "angle"}
	elseif what == "wellspring" then
		return {"speed"}
	end
	if what == "charge" or what == "pickup" or what == "unlock" then
		return {"category"}
	elseif what == "eol" then
		return {"target"}
	elseif what == "home" then
		return {"visible"}
	elseif what == "spring" then
		return {"xForce", "yForce"}
	elseif what == "teleport" then
		return {"momentum", "visible"}
	end
	if what == "baddie" or what == "boss" then
		return {"width", "height", "species"}
	elseif what == "floater" then
		return {"speed", "range", "resolve"}
	elseif what == "npc" then
		return {"speed", "range"}
	elseif what == "signpost" then
		return {"dialogWidth"}
	end
	return {}
end

function editor:fillArea(inputX, inputY, rows, spread)
	print("Starting fillArea")
	local startX = inputX or 0
	local startY = inputY or 0
	rows = rows or 8
	spread = spread or 4096
	for i=1, rows do	-- we work in downwards in rows
		print("Starting row " .. i)
		local lastObject = {
			h = 1,
			y = startY,
			w = 1,
			x = startX
		}
		while lastObject.x < (startX + spread) do
			local newObject = {}
			newObject.h = math.random(1, 10)
			newObject.y = (math.random(0, (10 - newObject.h)) * 32) + (i * 320) + startY
			newObject.w = math.random(1, 24)
			newObject.x = (math.random(0, 6) * 32) + (lastObject.x + (lastObject.w * 32))
			print("Values are "..newObject.h.." "..newObject.y.." "..newObject.w.." "..newObject.x)
			self:newGeometry(self.brush, newObject.x, newObject.y, newObject.w, newObject.h)
			lastObject.h = newObject.h
			lastObject.y = newObject.y
			lastObject.w = newObject.w
			lastObject.x = newObject.x
		end
	end
end

function editor:deleteEnt()
	if self.select > 0 then
		table.remove(ent[self.set][self.brush], self.select)
		setIDs(self.set, self.brush)
	end
	self.select = 0
end

function editor:newEnt(x, y, precision)
	x = x or (ent.actor.avatar[1].body:getX() - 32)
	y = y or (ent.actor.avatar[1].body:getY() + 32)
	precision = precision or 32
	x = modDown(x, precision)
	y = modDown(y, precision)
	if self.set == "geometry" then
		self:newGeometry(self.brush, x, y)
	elseif self.set == "sensor" then
		self:newSensor(self.brush, x, y)
	else
		self:newActor(self.brush, x, y)
	end
end

function editor:newGeometry(what, x, y, w, h)
	self.select = #ent.geometry[self.brush] + 1
	w = w or 2
	h = h or 2
	if what == "barrier" then
		ent.geometry.barrier[self.select] = barrier:new(x, y)
	elseif what == "blockMaker" then
		ent.geometry.blockMaker[self.select] = blockMaker:new(x, y)
	elseif what == "breakable" then
		ent.geometry.breakable[self.select] = breakable:new(x, y)
	elseif what == "clip" then
		ent.geometry.clip[self.select] = clip:new(x, y, w, h, 32)
	elseif what == "crate" then
		ent.geometry.crate[self.select] = crate:new(x, y)
	elseif what == "platform" then
		ent.geometry.platform[self.select] = platform:new(x, y)
	elseif what == "ramp" then
		ent.geometry.ramp[self.select] = ramp:new(x, y)
	elseif what == "solidBlock" then
		ent.geometry.solidBlock[self.select] = solidBlock:new(x, y, w, h, 32, true)
	elseif what == "spikes" then
		ent.geometry.spikes[self.select] = spikes:new(x, y, w, h, 32, 0, 1)
	elseif what == "wellspring" then
		ent.geometry.wellspring[self.select] = wellspring:new(x, y)
	elseif what == "water" then
		ent.geometry.water[self.select] = water:new(x, y, w, h, 32)
	else
		ent.geometry.terrain[self.select] = terrain:new(x, y, w, h, 32, true)
	end
	ent.geometry[what][self.select].id = self.select
	print("Created new " .. what .. " id " .. self.select .. " at " .. x .. " x " .. y)
end

function editor:newSensor(what, x, y)
	self.select = #ent.sensor[self.brush] + 1
	if what == "charge" then
		ent.sensor.charge[self.select] = charge:new(x, y, "health")
	elseif what == "eol" then
		ent.sensor.eol[self.select] = eol:new(x, y, 1)
	elseif what == "home" then
		ent.sensor.home[self.select] = home:new(x, y, {64, 160, 256, 352}, true)
	elseif what == "spring" then
		ent.sensor.spring[self.select] = spring:new(x, y, 0, -64)
	elseif what == "teleport" then
		ent.sensor.teleport[self.select] = teleport:new(x, y, 8160, 8192, 1, true)
	elseif what == "unlock" then
		ent.sensor.unlock[self.select] = unlock:new(x, y, "lightning", 1)
	else
		ent.sensor.pickup[self.select] = pickup:new(x, y, "will")
	end
	ent.sensor[what][self.select].id = self.select
	print("Created new " .. what .. " id " .. self.select .. " at " .. x .. " x " .. y)
end

function editor:newActor(what, x, y)
	self.select = #ent.actor[self.brush] + 1
	if what == "npc" then
		ent.actor.npc[self.select] = npc:new(x, y)
	elseif what == "floater" then
		ent.actor.floater[self.select] = floater:new(x, y)
	elseif what == "signpost" then
		ent.actor.signpost[self.select] = signpost:new(x, y)
	elseif what == "boss" then
		ent.actor.boss[self.select] = boss:new(x, y)
	else
		ent.actor.baddie[self.select] = baddie:new(x, y)
	end
	ent.actor[what][self.select].id = self.select
	print("Created new " .. what .. " id " .. self.select .. " at " .. x .. " x " .. y)
end

function editor:moveObject(newX, newY)
	local target = ent[self.set][self.brush][self.select]
	local currentX = target.body:getX() + newX
	local currentY = target.body:getY() + newY
	target.body:setPosition(currentX, currentY)
	print("Moved to " .. currentX .. " x " .. currentY)
end

function editor:scaleObject(changeInWidth, changeInHeight)
	local target = ent.geometry[self.brush][self.select]
	local newWidth = target.width + changeInWidth
	local newHeight = target.height + changeInHeight
	if newWidth > 0 and newHeight > 0 then
		target.width = newWidth
		target.height = newHeight
		print("Scaled to " .. target.width .. " by " .. target.height)
		-- we have to change the X so the shape doesn't grow from the middle
		local currentX, currentY = target.body:getPosition()
		currentX = currentX + (changeInWidth / 2)
		currentY = currentY + (changeInHeight / 2)
		target.body:setPosition(currentX, currentY)
		print("Moved to " .. currentX .. " x " .. currentY)
		-- we have to destroy and remake the shape to resize it
		target:makeShape()
		-- some objects need to regenerate their images
		if target.imageSize then
			target.imageSize = makeImage(target.width, target.height, target.name)
		end
	else
		dialog.text = "You cannot reduce that dimension any further."
	end
end

function editor:changeSensorParams(which, value)
	local target = ent.sensor[self.brush][self.select]
	if what == "charge" then
	elseif what == "eol" then
	elseif what == "home" then
	elseif what == "spring" then
	elseif what == "teleport" then
	elseif what == "unlock" then
		if which == "category" then
			target.category = nextInList(target.category, {"lightning", "bomb", "boomerang", "jump"})
			loadStaticImg("pickups", "unlock_"..target.category)
		end
	else	-- pickup
		if which == "category" then
			target.category = nextInList(target.category, {"attack", "cooldown", "health", "jumps", "will"})
			loadStaticImg("pickups", "pickup_"..target.category)
		end
	end
	print("Parameter " .. which .. " changed by " .. value .. " on a sensor")
end

function editor:changeActorParams(which, value)
	local target = ent.actor[self.brush][self.select]
	-- do stuff
	print("Parameter " .. which .. " changed by " .. value .. " on an actor")
end

function editor:changeBrush(number)
	self.brushIndex = self.brushIndex + number
	if self.brushIndex > #self.brushList[self.set] then
		self.brushIndex = 1
	elseif self.brushIndex < 1 then
		self.brushIndex = #self.brushList[self.set]
	end
	self.brush = self.brushList[self.set][self.brushIndex]
	self.select = #ent[self.set][self.brush]
	print("Brush = " .. self.brush)
end

function editor:changeBrushSet()
	self.set = nextInList(self.set, {"geometry", "sensor", "actor"})
	print("Set = " .. self.set)
	self:changeBrush(0)	-- also need to refresh the current brush
end

function editor:changeSelection(number)
	self.select = self.select + number
	local count = #ent[self.set][self.brush]
	if count > 0 then
		if self.select > count then
			self.select = 1
		elseif self.select < 1 then
			self.select = count
		end
	else
		self.select = 0
	end
	print("Selection = " .. self.select)
end

function editor:changeSelectionViaMouse(x,y)
	print("Changing selection via mouse at " .. x .. " by " .. y)
	local count = #ent[self.set][self.brush]
	if count > 0 then
		for i=1, count do
			target = ent[self.set][self.brush][i]
			if testOverlap(x,y,4,4, target.body:getX()-(target.width/2),target.body:getY()-(target.height/2),target.width,target.height) then
				self.select = i
				print("Selection found, ID " .. i)
			end
		end
	else
		dialog:setText("There is nothing in this brush set to select.")
		dialog:setResponses()
	end
end
