-- Player 
-- Added 2010-11-08 by Thomas Holden
-- Edited 2011-04-15 by Thomas Holden
-- Stores and manages the player information

player = class:new()

function player:init(id, keys, species, companion, team)
	self.id = id or 1
	self.species = species or "elk"
	self.companion = companion or "fish"
	self.team = team or 1	-- which team you're on (int)
	self.health = 3			-- damage you can take before dying
	self.will = 3			-- todo: shots you can fire successively without recharging (consider dropping)
	self.cooldown = 0		-- how long till you can attack again (in seconds)
	self.selected = "lunge"	-- which move is selected
	self.bucket = 0			-- used in team mode for score
	-- keyboard bindings
	self.key = {
		up = keys[1] or "up",
		down = keys[2] or "down",
		left = keys[3] or "left",
		right = keys[4] or "right",
		attack = keys[5] or "rctrl",
		action = keys[6] or "rshift",
		pause = keys[7] or "backspace"
	}
	-- activity history
	self.history = {
		wins = 0,
		losses = 0,
		score = 0,
		kills = 0,
		deaths = 0,
		jumps = 0
	}
	-- player long-term maxes
	self.max = {
		health = 3,
		will = 3,		-- unused...
		jumps = 1,		-- how many jumps you can do
		speed = 600		-- player's max running speed
	}
	-- player moves
	self.moves = {
		lunge = {
			power = 1,		-- the power of your default lunging attack
			cooldown = 3,	-- in seconds
			force = 200,	-- how much force it has
			count = 0		-- how many times this move has been used
		},
		lightning = {
			power = 0,
			cooldown = 3,
			speed = 720,
			count = 0,
		},
		bomb = {
			power = 0,
			cooldown = 4,
			duration = 5,
			count = 0
		},
		boomerang = {
			power = 0,
			cooldown = 3,
			range = 200,
			count = 0
		}
	}
	-- the items you've picked up (elsewhere called pickups)
	self.items = {
		health = 0,
		will = 0,
		jumps = 0,
		attack = 0,
		cooldown = 0
	}
	self:positionStats()
	print("Player ID " .. self.id .. " created")
end

function player:update(dt)
	local left = love.keyboard.isDown(self.key.left) or joystick.pressed[self.key.left]
	local right = love.keyboard.isDown(self.key.right) or joystick.pressed[self.key.right]
	if right and not left then
		ent.actor.avatar[self.id].direction = 1
		ent.actor.avatar[self.id]:run()
	elseif left and not right then
		ent.actor.avatar[self.id].direction = -1
		ent.actor.avatar[self.id]:run()
	end
	if self.cooldown > 0 then
		self.cooldown = self.cooldown - dt
	end
	if self.bucket > 0 then
		self.bucket = self.bucket - (5 * dt)
	end
end

function player:draw()
	-- unused
end

function player:keypressed(key)
	if self.health > 0 then
		if key == self.key.up then
			ent.actor.avatar[self.id]:jump()
			self.history.jumps = self.history.jumps + 1
		elseif key == self.key.down then
			ent.actor.avatar[self.id]:duck()
		elseif key == self.key.attack then
			if self.cooldown <= 0 and self.moves[self.selected].power > 0 then		-- keep players from spamming
				self:attack()
			else
				ent.actor.companion[self.id].flash = 2
				print("Cannot attack yet")
			end
		elseif key == self.key.action then
			self:cycleMoves()
		end
	end
end

function player:positionStats()
	-- for viewing player stats
	self.viewstats = {
		x = screen.halfWidth - 416,
		y = screen.halfHeight - 256,
		page = 1
	}
	-- recalc viewstats calculate coords if needed
	if (self.id % 2) == 0 then
		self.viewstats.x = screen.halfWidth + 160
	end
	if self.id > 2 then
		self.viewstats.y = screen.halfHeight
	end
end

function player:drawStats()
	local drawX = self.viewstats.x
	local drawY = self.viewstats.y
	-- draw bg
	love.graphics.draw(staticImg.menu_top, drawX, drawY - 16)
	for i=1, 6 do
		love.graphics.draw(staticImg.menu_tile, drawX, drawY + (32 * i) - 16)
	end
	love.graphics.draw(staticImg.menu_foot, drawX, drawY + 208)
	-- draw text
	love.graphics.printf("Player " .. self.id .. " (" .. self.species .. ")", drawX, drawY, 256, "center")
	if self.viewstats.page == 1 then
		local stat_headings = {"Health", "Jumps", "Attack", "Cooldown", "Team", "Bucket"}
		for i=1, #stat_headings do
			love.graphics.print(stat_headings[i]..":", drawX + 32, drawY + (i * 32))
		end
		local newX = drawX + 120
		love.graphics.print(self.health.." / "..self.max.health, newX, drawY + 32)
		love.graphics.print(ent.actor.avatar[self.id].jumpsLeft.." / "..self.max.jumps, newX, drawY + 64)
		love.graphics.print(self.selected.." (Level "..self.moves[self.selected].power..")", newX, drawY + 96)
		love.graphics.print(round(self.cooldown,2).." / "..self.moves[self.selected].cooldown, newX, drawY + 128)
		love.graphics.print(self.team, newX, drawY + 160)
		love.graphics.print(self.bucket, newX, drawY + 192)
	elseif self.viewstats.page == 2 then
		local hist_headings = {"wins", "losses", "score", "deaths", "kills", "jumps"}
		for i=1, #hist_headings do
			local text = properCase(hist_headings[i])
			love.graphics.print(text..":", drawX + 32, drawY + (i * 32))
			love.graphics.print(self.history[hist_headings[i]], drawX + 120, drawY + (i * 32))
		end
	else
		love.graphics.print("Items Picked Up:", drawX + 32, drawY + 32)
		local item_headings = {"health", "will", "jumps", "attack", "cooldown"}
		for i=1, #item_headings do
			local text = properCase(item_headings[i])
			love.graphics.print(text..":", drawX + 32, drawY + (i * 32) + 32)
			love.graphics.print(self.items[item_headings[i]], drawX + 120, drawY + (i * 32) + 32)
		end
	end
end

function player:attack()
	if self.selected == "lunge" then
		ent.actor.avatar[self.id]:lunge(self.moves.lunge.force)
	else
		local spawnX = ent.actor.avatar[self.id].body:getX() + (48 * ent.actor.avatar[self.id].direction)
		local spawnY = ent.actor.avatar[self.id].body:getY()
		if self.selected == "lightning" then
			if ent.actor.avatar[self.id].direction < 0 then
				spawnX = spawnX - 64	-- extra offset because lightning is very wide
			end
			-- parent, x, y, direction, strength, speed
			table.insert(ent.projectile.lightning, lightning:new(self.id, spawnX, spawnY, ent.actor.avatar[self.id].direction, self.moves.lightning.power, self.moves.lightning.speed))
		elseif self.selected == "bomb" then
			-- parent, x, y, xdirection, ydirection, strength, countdown, duration
			local xdir = 32 * ent.actor.avatar[self.id].direction
			table.insert(ent.projectile.bomb, bomb:new(self.id, spawnX, spawnY, xdir, -64, self.moves.bomb.power, 3, self.moves.bomb.duration))
		elseif self.selected == "boomerang" then
			-- parent, x, y, xdirection, ydirection, strength, range
			local xdir = 256 * ent.actor.avatar[self.id].direction
			table.insert(ent.projectile.boomerang, boomerang:new(self.id, spawnX, spawnY, xdir, 0, self.moves.boomerang.power))
		end
		print("Created new " .. self.selected .. " at " .. spawnX .. " by " .. spawnY)
		if #ent.projectile[self.selected] > 24 then		-- keep the number of projectiles down by removing the oldest one
			table.remove(ent.projectile[self.selected], 1)
		end
	end
	self.moves[self.selected].count = self.moves[self.selected].count + 1
	self.cooldown = self.moves[self.selected].cooldown
	print("Player ID " .. self.id .. " cooldown set to " .. self.cooldown)
end

function player:cycleMoves()	-- cycles through attack moves, setting self.selected
	self.selected = nextInList(self.selected, {"lunge", "lightning", "bomb", "boomerang"})
	if self.moves[self.selected].power == 0 then
		self:cycleMoves()
	else
		print("Changed selected move to " .. self.selected)
	end
end

function player:setHealth(number, cause)	-- works with negatives
	if (self.health + number) <= self.max.health then
		self.health = self.health + number
	end
	print("Player ID " .. self.id .. " health set to " .. self.health)
	if self.health <= 0 then
		ent.actor.avatar[self.id]:kill()
		self.history.deaths = self.history.deaths + 1
		print("Player ID " .. self.id .. " killed and has " .. self.history.deaths .. " recorded deaths")
	elseif number < 0 then
		ent.actor.companion[self.id].flash = 2	-- indicate that we have taken damage
	end
end
