-- utils
-- Added 2010-12-18 by Thomas Holden
-- Edited 2011-04-15 by Thomas Holden
-- Creates useful little utility functions

-- physics-related functions

function add(shape1, shape2, collision)
	if shape1.collide then
		shape1:collide(shape2)
	end
	if shape2.collide then
		shape2:collide(shape1)
	end
end

function persist(shape1, shape2, collision)
	if shape1.persist then
		shape1:persist(shape2)
	end
	if shape2.persist then
		shape2:persist(shape1)
	end
end

function remove(shape1, shape2, collision)
	if shape1.remove then
		shape1:remove(shape2)
	end
	if shape2.remove then
		shape2:remove(shape1)
	end
end

function testOverlap(x1,y1,w1,h1, x2,y2,w2,h2)	-- check if two objects overlap
	-- by default the second object is the screen
	x2 = x2 or camera.x - ((screen.width - (screen.width * camera.zoom.x * camera.zoom.modifier)) / 2)
	y2 = y2 or camera.y - ((screen.height - (screen.height * camera.zoom.y * camera.zoom.modifier)) / 2)
	w2 = w2 or screen.width / (camera.zoom.x * camera.zoom.modifier)
	h2 = h2 or screen.height / (camera.zoom.y * camera.zoom.modifier)
	return not (x1+w1 < x2 or x2+w2 < x1 or y1+h1 < y2 or y2+h2 < y1)
end

function negateGravity(body)
	local mass = body:getMass()
	local gravX, gravY = world:getGravity()
	local meter = world:getMeter()
	body:applyForce(-gravX * mass / meter, -gravY * mass / meter)
end

function setIDs(set, subset)
	if #ent[set][subset] > 0 then
		for i=1, #ent[set][subset] do
			ent[set][subset][i].id = i
			print(subset .. " ID " .. i .. " set")
		end
	end
end

function physicsRemoval(self)
	self.shape:setMask(1,2,3,4)
	self.body:setY(killdepth)
	--self.shape:destroy()
	--self.body:destroy()
end

-- math-related functions

function round(what, shift)
	local shift = 10 ^ (shift or 0)
	return math.floor(what * shift + 0.5 ) / shift
end

function modDown(number, modNumber)
	modNumber = modNumber or 32
	return number - (number % modNumber)
end

function modUp(number, modNumber)
	modNumber = modNumber or 32
	return number - (number % modNumber) + modNumber
end

-- graphics-related functions

function defaultVideoSettings()
	screen = {
		width = 1024,
		height = 640,
		halfWidth = 512,
		halfHeight = 320,
		fullscreen = false,
		vsync = false,
		fsaa = 0
	}
	showFPS = false
	editHelp = true
	editLens = false
end

function defaultSoundSettings()
	volume = {
		master = .5,
		music = .5,
		sfx = .5
	}
end

function reloadGraphics()
	print("Reloading graphics")
	love.graphics.setMode(screen.width, screen.height, screen.fullscreen, screen.vsync, screen.fsaa)
	screen.halfWidth = screen.width / 2		-- probably need to recalc halfs
	screen.halfHeight = screen.height / 2
end

function setDefaultColor()
	love.graphics.setColor(255, 255, 255, 255)
end

function loadStaticImg(subfolder, which)
	if staticImg[which] == nil then
		staticImg[which] = newPaddedImage("media/images/"..subfolder.."/"..which..".png")
		print("Static image " .. which .. " loaded")
	end
end

function insertGeneratedImg(subgroup, size, image)
	if generatedImg[subgroup][size] == nil then
		generatedImg[subgroup][size] = image
		print(size .. " image inserted for " .. subgroup)
	end
end

function loadAnimSheet(subfolder, which)
	if animSheet[which] == nil then
		animSheet[which] = newPaddedImage("media/images/"..subfolder.."/"..which..".png")
		print("Anim sheet " .. which .. " loaded")
	end
	return animSheet[which]
end

function pulsateAlpha(alpha, change, dt, maximum, mininimum)
	maximum = maximum or 250
	mininimum = mininimum or 128
	if alpha >= maximum and change > 1 then
		change = -change
	elseif alpha <= mininimum and change < 1 then
		change = -change
	end
	alpha = alpha + (change * dt)
	return alpha, change
end

function rotateImage(angle, dt)
	angle = angle + (dt * 360)
	if angle > 360 then
		angle = angle - 360
	end
	return angle
end

function newPaddedImage(source)		-- returns a power of two image 
	if type(source) == "string" then
		source = love.image.newImageData(source)
	end
	local width = source:getWidth()
	local height = source:getHeight()
	local powerwidth = math.pow(2, math.ceil(math.log(width)/math.log(2)))
	local powerheight = math.pow(2, math.ceil(math.log(height)/math.log(2)))
	if powerwidth ~= width or powerheight ~= height then
		local padded = love.image.newImageData(powerwidth, powerheight)
		padded:paste(source, 0, 0)
		print("Returning padded image")
		return love.graphics.newImage(padded)
	end
	return love.graphics.newImage(source)
end

function makeImage(width, height, name)
	local image = generateTexture(width, height, name)
	local imageSize = width .. "x" .. height
	insertGeneratedImg(name, imageSize, image)
	return imageSize
end

function generateTexture(width, height, category)
	width = width or 64
	height = height or 64
	category = category or "terrain"
	local location = "media/images/world/" .. category .. "/"
	local raw = love.image.newImageData(width, height)
	if category == "ramp" then
		if width <= 32 and height <= 32 then
			local body = love.image.newImageData(location.."single.png")
			raw:paste(body, 0, 0, 0, 0, 32, 32)
		else
			local face = love.image.newImageData(location .. "face.png")
			local side = love.image.newImageData(location .. "side.png")
			local foot = love.image.newImageData(location .. "foot.png")
			local corner = love.image.newImageData(location .. "corner.png")
			local corner90 = love.image.newImageData(location .. "90.png")
			local cornerTop = love.image.newImageData(location .. "top.png")
			local filler = love.image.newImageData(location .. "filler.png")
			for i=0, width / 32 do
				for j=0, height / 32 do
					-- note: the image is flipped later, so we can do all our ifs for \ slopes
					if i == 0 then
						if j == 0 then
							raw:paste(cornerTop, 0, 0, 0, 0, 32, 32)
						elseif j == (height / 32) - 1 then
							raw:paste(corner90, 0, height - 32, 0, 0, 32, 32)
						else 
							raw:paste(side, 0, j * 32, 0, 0, 32, 32)
						end
					elseif i == (width / 32) - 1 and j == (height / 32) - 1 then
						raw:paste(corner, width - 32, height - 32, 0, 0, 32, 32)
					elseif i == j then
						raw:paste(face, i * 32, j * 32, 0, 0, 32, 32)
					elseif j == (height / 32) - 1 then
						raw:paste(foot, i * 32, j * 32, 0, 0, 32, 32)
					elseif i == (j + 1) then
						raw:paste(filler, i * 32, j * 32 + 16, 0, 0, 16, 16)
					else
						-- empty space
					end
				end
			end	-- end of for loops
		end
	else
		if width <= 32 and height <= 32 then
			local body = love.image.newImageData(location.."single.png")
			raw:paste(body, 0, 0, 0, 0, 32, 32)
		elseif width <= 32 then
			local top = love.image.newImageData(location.."top_cap.png")
			local body = love.image.newImageData(location.."single_vertical.png")
			local bottom = love.image.newImageData(location.."foot_cap.png")
			for h=0, (height / 32) - 1 do
				if h == 0 then
					raw:paste(top, 0, 0, 0, 0, 32, 32)
				elseif h == (height / 32) - 1 then
					raw:paste(bottom, 0, h * 32, 0, 0, 32, 32)
				else
					raw:paste(body, 0, h * 32, 0, 0, 32, 32)
				end
			end
		elseif height <= 32 then
			local bodyLeft = love.image.newImageData(location.."left_cap.png")
			local body = love.image.newImageData(location.."single_horizontal.png")
			local bodyRight = love.image.newImageData(location.."right_cap.png")
			for w=0, (width / 32) - 1 do
				if w == 0 then
					raw:paste(bodyLeft, 0, 0, 0, 0, 32, 32)
				elseif w == (width / 32) - 1 then
					raw:paste(bodyRight, w * 32, 0, 0, 0, 32, 32)
				else
					raw:paste(body, w * 32, 0, 0, 0, 32, 32)
				end
			end
		else
			local topLeft = love.image.newImageData(location.."top_left.png")
			local top = love.image.newImageData(location.."top.png")
			local topRight = love.image.newImageData(location.."top_right.png")
			local bodyLeft = love.image.newImageData(location.."left_wall.png")
			local body = love.image.newImageData(location.."middle.png")
			local bodyRight = love.image.newImageData(location.."right_wall.png")
			local footLeft = love.image.newImageData(location.."foot_left.png")
			local foot = love.image.newImageData(location.."foot.png")
			local footRight = love.image.newImageData(location.."foot_right.png")
			for h=0, (height / 32) - 1 do
				for w=0, (width / 32) - 1 do
					if h == 0 then	-- the top
						if w == 0 then
							raw:paste(topLeft, 0, 0, 0, 0, 32, 32)
						elseif w == (width / 32) - 1 then
							raw:paste(topRight, (width - 32), 0, 0, 0, 32, 32)
						else
							raw:paste(top, (w * 32), 0, 0, 0, 32, 32)
						end
					elseif h == (height / 32) - 1 then	-- the bottom
						if w == 0 then
							raw:paste(footLeft, 0, (height - 32), 0, 0, 32, 32)
						elseif w == (width / 32) - 1 then
							raw:paste(footRight, (width - 32), (height - 32), 0, 0, 32, 32)
						else
							raw:paste(foot, (w * 32), (height - 32), 0, 0, 32, 32)
						end
					else
						if w == 0 then
							raw:paste(bodyLeft, 0, (h * 32), 0, 0, 32, 32)
						elseif w == (width / 32) - 1 then
							raw:paste(bodyRight, (width - 32), (h * 32), 0, 0, 32, 32)
						else
							raw:paste(body, (w * 32), (h * 32), 0, 0, 32, 32)
						end
					end
				end
			end	-- ends of for loops
		end
	end
	return newPaddedImage(raw)
end

-- gui-related functions

function getDefaultBindings()
	-- the default controls for all four players
	local defaultbindings = {
		{ "up", "down", "left", "right", "rctrl", "rshift", "backspace" },
		{ "w", "s", "a", "d", "f", " ", "escape" },
		{ "i", "k", "j", "l", ";", "ralt", "/" },
		{ "kp8", "kp2", "kp4", "kp6", "kp+", "kpenter", "numlock" }
	}
	return defaultbindings
end

function checkBindings(keyString)
	for i=1,4 do		-- go thru players
		for j=1,7 do	-- go thru keys
			if bindings[i][j] == keyString then
				return i, j
			end
		end
	end
	return 0, 0
end

function countFiles(whereAt)
	local total = 0
	for i,file in pairs(love.filesystem.enumerate(whereAt)) do
		if file:find('%.lua$') then
			total = total + 1
		end
	end
	print("Files counted (" .. total .. " total)")
	return total
end

function listFiles(whereAt)
	local array = {}
	for i,file in pairs(love.filesystem.enumerate(whereAt)) do
		if file:find('%.lua$') then
			local filename = string.sub(file, 1, -5)		-- make a temp name from the first char to the fifth to last one 
			filename = underscoresToSpaces(filename)		-- now make it pretty
			--filename = string.upper(string.sub(filename, 1, 1)) .. string.lower(string.sub(filename, 2))
			table.insert(array, filename)
		end
	end
	print("Files listed (" .. #array .. " total)")
	return array
end

function toText(incoming)
	if incoming == nil then
		return "nil"
	elseif type(incoming) == "string" then
		return tostring(incoming)
	elseif type(incoming) == "boolean" then	-- type(incoming) == boolean
		return boolToString(incoming)
	end
	return incoming
end

function boolToString(newBoolean)
	if newBoolean then
		return "true"
	end
	return "false"
end

function testStrings(goal, options)
	for i=1, #options do
		if goal == tostring(options[i]) then
			return true
		end
	end
	return false
end

function nextInList(current, list)
	for i=1, #list do
		if current == list[i] then
			if i == #list then
				return list[1]
			else
				return list[i + 1]
			end
		end
	end
	return current
end

function prevInList(current, list)
	for i=1, #list do
		if current == list[i] then
			if i == 1 then
				return list[#list]
			else
				return list[i - 1]
			end
		end
	end
	return current
end

constWords = {
	[" "] = "Space",
	["!"] = "Exclamation Mark",
	["\""] = "Double Quote",
	["#"] = "Hash",
	["$"] = "Dollar Sign",
	["&"] = "ampersand",
	["'"] = "Single Quote",
	["("] = "Left Parenthesis",
	[")"] = "Right Parenthesis",
	["*"] = "Asterisk",
	["+"] = "Plus",
	[","] = "Comma",
	["-"] = "Hyphen",
	["."] = "Period",
	["/"] = "Slash",
	[":"] = "Colon",
	[";"] = "Semicolon",
	["<"] = "Less-than",
	["="] = "Equal",
	[">"] = "Greater-than",
	["?"] = "Question Mark",
	["@"] = "At Sign",
	["["] = " Left Bracket",
	["\\"] = "Backslash",
	["]"] = "Right Bracket",
	["^"] = "Caret",
	["_"] = "Underscore",
	["`"] = "Grave Accent",
	["rshift"] = "Right Shift",
	["lshift"] = "Left Shift",
	["rctrl"] = "Right Control",
	["lctrl"] = "Left Control",
	["ralt"] = "Right Alt",
	["lalt"] = "Left Alt",
	["rmeta"] = "Right Meta",
	["lmeta"] = "Left Meta",
	["rsuper"] = "Right Super",
	["lsuper"] = "Left Super",
}

function keyWord(k)		-- special thanks to slime for rewriting this
	if k and type(k) == "string" and #k > 0 then
		return constWords[k] or properCase(k)
	end
	return properCase(k)
end

function properCase(word)
	if word and type(word) == "string" then
		return word:sub(1,1):upper() .. word:sub(2)
	end
	return word
end

function underscoresToSpaces(word)
	if word and type(word) == "string" then
		return string.gsub(word, "_", " ")
	end
	return word
end

function spacesToUnderscores(word)
	if word and type(word) == "string" then
		return string.gsub(word, " ", "_")
	end
	return word
end

function printInit(self, x, y)
	print("New " .. self.name .. " created at " .. x .. " by " .. y)
	-- ID hasn't been set yet, which is kind of a problem
end

-- gamemode-related function

function versusSetStats(who, amount)
	amount = amount or 1
	players[who].moves.lightning.power = amount
	players[who].moves.bomb.power = amount
	players[who].moves.boomerang.power = amount
end

function versusRecordResults()
	for i=1, multiplayer do
		if players[i].health > 0 then	-- we can safely assume that this guy who won
			players[i].history.wins = players[i].history.wins + 1
		else
			players[i].history.losses = players[i].history.losses + 1
		end
	end
end

function teamRecordResults(winner)
	winner = winner or 1
	for i=1, multiplayer do
		if players[i].team == winner then
			players[i].history.wins = players[i].history.wins + 1
		else
			players[i].history.losses = players[i].history.losses + 1
		end
	end
end

function teamCheckLiving()
	local teams = {false, false}	-- assume both teams are dead
	for i=1, multiplayer do
		if players[i].health > 0 then
			teams[players[i].team] = true
		end
	end
	return teams[1], teams[2]
end
